# Generated by roxygen2: do not edit by hand

S3method(plot,nullhespdiv)
S3method(print,hespdiv)
S3method(print,nullhespdiv)
export(blok3d)
export(change_base)
export(create_gif)
export(cross_comp)
export(dendro)
export(depend_splits)
export(get_data)
export(group_effect)
export(hespdiv)
export(hsa)
export(hsa_detailed)
export(hsa_quant)
export(hsa_sample_constrained)
export(nulltest)
export(plot_cs_hsa)
export(plot_hespdiv)
export(plot_hsa)
export(plot_hsa_q)
export(poly_scheme)
export(polypop)
export(remove_splits)
export(taxon_effect)
importFrom(RColorBrewer,brewer.pal.info)
importFrom(future,availableCores)
importFrom(future,multisession)
importFrom(future,plan)
importFrom(future.apply,future_lapply)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_colourbar)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,scale_size_area)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_set)
importFrom(ggplot2,theme_void)
importFrom(ggrepel,geom_label_repel)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,chull)
importFrom(grDevices,colors)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.new)
importFrom(grDevices,dev.off)
importFrom(grDevices,dev.set)
importFrom(grDevices,png)
importFrom(graphics,boxplot)
importFrom(graphics,hist)
importFrom(graphics,layout)
importFrom(graphics,lines)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(grid,grid.draw)
importFrom(grid,grid.grabExpr)
importFrom(grid,grid.newpage)
importFrom(gridExtra,arrangeGrob)
importFrom(gridGraphics,grid.echo)
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,ends)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,layout_as_tree)
importFrom(igraph,vcount)
importFrom(magick,image_animate)
importFrom(magick,image_read)
importFrom(magick,image_write)
importFrom(pracma,poly_center)
importFrom(pracma,polyarea)
importFrom(rgl,identify3d)
importFrom(rgl,lines3d)
importFrom(rgl,open3d)
importFrom(rgl,par3d)
importFrom(rgl,persp3d)
importFrom(rgl,plot3d)
importFrom(rgl,points3d)
importFrom(rgl,polygon3d)
importFrom(rgl,rgl.ids)
importFrom(rgl,rgl.pop)
importFrom(rgl,rgl.snapshot)
importFrom(rgl,rotate3d)
importFrom(rgl,rotationMatrix)
importFrom(rgl,text3d)
importFrom(rgl,view3d)
importFrom(rlang,.data)
importFrom(scales,alpha)
importFrom(stats,aggregate)
importFrom(stats,density)
importFrom(stats,na.omit)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(stats,spline)
importFrom(utils,combn)
importFrom(viridis,scale_color_viridis)
useDynLib(hespdiv, R_point_in_polygon_sp)
