#'
if(getRversion() >= "2.15.1"){

  utils::globalVariables(c("S.crit", ".slicer", ".comp", "c.sign", ".minormax", ".which_minormax",
                           "samp.xy", "rims", "poly.obj", "generalize.f", "e", "n.split.pts",
                           "dst.pts", "trace.object", "pnts.col", "xy.dat", "study.pol.viz",
                           "S.rel.crit", "c.Q.crit", "trace.level", ".get_ids_str", "N.crit",
                           "N.rel.crit", "N.loc.crit", "N.loc.rel.crit", "S.cond", "str.split.quals",
                           "sd", "c.splits", "c.X.knots", "c.Y.knots", "c.max.iter.no",
                           "c.fast.optim", "c.corr.term", "c.crit.improv", "Q.crit", "splits","testid"
  ))
  }

